import numpy as np
import pandas as pd


def create_date(min_year=2000, max_year=2020):
    """Create a random date.

    Parameters
    ----------
    min_year : int, optional
        The minimum possible year, by default 2000.
    max_year : int, optional
        The maximum possible year, by default 2020.

    Returns
    -------
    pandas.Timestamp
        A random date.
    """
    year = np.random.randint(min_year, max_year, size=1)[0]
    month = np.random.randint(1, 13, size=1)[0]
    if month == 2:
        max_day = 28
    elif month in [1, 3, 5, 7, 8, 10, 12]:
        max_day = 31
    else:
        max_day = 30
    day = np.random.randint(1, 29, size=1)[0]
    date = pd.to_datetime("-".join([str(year), str(month).zfill(2), str(day).zfill(2)]))
    return date



"""
Create a business with fake data.
"""
import numpy as np
import pandas as pd
from faker import Faker

from pydatafaker.utilities import create_date


def create_vendor_table(n=100):
    """Create a table of fake vendors.

    Parameters
    ----------
    n : int, optional
        The number of vendors to create, by default 100.

    Returns
    -------
    pandas.DataFrame
        A dataframe containing: vendor_id, vendor_name, and vendor_address.
    """
    fake = Faker()
    x = pd.DataFrame(
        {
            "vendor_id": ["vendor_" + str(i).zfill(5) for i in range(1, n + 1)],
            "vendor_name": [fake.company() for _ in range(n)],
            "vendor_description": [fake.catch_phrase() for _ in range(n)],
            "address": [fake.address() for _ in range(n)],
            "phone": [fake.phone_number() for _ in range(n)],
            "email": [fake.safe_email() for _ in range(n)]
        }
    )
    return x


def create_employee_table(vendor_id, n=100):
    """Create a table of fake employees.

    Parameters
    ----------
    vendor_id : list
        A list of vendor ids to randomly assign to employees.
    n : int, optional
        The number of employees to create, by default 100.

    Returns
    -------
    pandas.DataFrame
        A dataframe containing: employee_id, employee_name, and vendor_id.
    """
    fake = Faker()
    x = pd.DataFrame(
        {
            "employee_id": ["vendor_" + str(i).zfill(5) for i in range(1, n + 1)],
            "employee_name": [fake.name() for _ in range(n)],
            "vendor_id": np.random.choice(vendor_id, replace=True, size=n),
        }
    )
    return x


def create_po_table(vendor_id, mean_po_amount=1_000_000, sd_po_amount=250_000, n=100):
    """Create a table of fake purchase orders (PO).

    Parameters
    ----------
    vendor_id : list
        A list of vendor ids to randomly assign to employees.
    n : int, optional
        The number of fake POs to create, by default 100.
    mean_po_amount : int, optional
        Mean value for normal distribution, by default 1_000_000.
    sd_po_amount : int, optional
        Standard deviation value for normal distribution, by default 250_000.

    Returns
    -------
    pandas.DataFrame
        A dataframe containing: po_id, vendor_id, and po_amount
    """
    x = pd.DataFrame(
        {
            "po_id": ["po_" + str(i).zfill(5) for i in range(1, n + 1)],
            "vendor_id": np.random.choice(vendor_id, replace=True, size=n),
            "po_amount": np.abs(
                np.random.normal(mean_po_amount, sd_po_amount, size=n)
            ).astype(int),
        }
    )
    return x


def create_invoice_table(
    po_table,
    mean_inv_line_amount=5_000,
    sd_inv_line_amount=4_000,
    min_year=2000,
    max_year=2020,
    n_invoice=250,
    n_line_item=5_000,
):
    """Create fake invoice and invoice line item tables.

    Parameters
    ----------
    po_table : pd.DataFrame
        The purchase order table created by create_po_table().
    mean_inv_line_amount : int, optional
        The mean value of invoice line items, by default 5_000.
    sd_inv_line_amount : int, optional
        The standard deviation of invoice line items, by default 4_000.
    min_year : int, optional
        The minimum possible year, by default 2000.
    max_year : int, optional
        The maximum possible year, by default 2020.
    n_invoice : int, optional
        The number of fake invoices to generate, by default 250.
    n_line_item : int, optional
        The number of fake invoice line items to generate, by default 5_000.

    Returns
    -------
    (pandas.DataFrame, pandas.DataFrame)
        A tuple of two dataframes: invoice_summary and invoice_line_items.
    """
    fake = Faker()
    invoice_ids = ["inv_" + str(i).zfill(5) for i in range(1, n_invoice + 1)]
    # invoice line items
    invoice_line_items = pd.DataFrame(
        {
            "invoice_id": np.random.choice(invoice_ids, replace=True, size=n_line_item),
            "invoice_line_id": [
                "line_item_" + str(i).zfill(9) for i in range(1, n_line_item + 1)
            ],
            "amount": np.abs(
                np.random.normal(
                    mean_inv_line_amount, sd_inv_line_amount, size=n_line_item
                )
            ).astype(int),
            "description": [fake.isbn10() for _ in range(n_line_item)],
        }
    )
    invoice_line_items = invoice_line_items.sort_values(by="invoice_id").reset_index(
        drop=True
    )
    # invoice summary
    invoice_summary = invoice_line_items.groupby("invoice_id")[["amount"]].sum()
    invoice_summary = invoice_summary.reset_index()
    invoice_summary["invoice_date"] = [
        create_date(min_year, max_year) for _ in range(n_invoice)
    ]
    invoice_summary["po_id"] = np.random.choice(
        po_table["po_id"], replace=True, size=n_invoice
    )
    invoice_summary = invoice_summary.merge(
        po_table[["po_id", "vendor_id"]], how="left", on="po_id"
    )
    return (invoice_summary, invoice_line_items)


def create_business(
    n_vendors=100,
    n_employees=100,
    mean_po_amount=1_000_000,
    sd_po_amount=250_000,
    mean_inv_line_amount=5_000,
    sd_inv_line_amount=4_000,
    min_year=2000,
    max_year=2020,
    n_invoice=250,
    n_line_item=5_000,
):
    """Create an entire fake business.

    A meta function that creates many tables with related entities. A
    dictionary is returned that contains the following tables:

    - vendor_table
    - po_table
    - invoice_summary_table
    - invoice_line_item_table
    - employee_table
    - rate_sheet_table (under development)
    - contract_table (under development)
    - timesheet_table (under development)

    The tables are related to eachother in such a way that all tables can be
    combined to access different data.

    Parameters
    ----------
    n_vendors : int, optional
        The number of fake vendors to create, by default 100.
    n_employees : int, optional
        The number of fake employees to create, by default 100.
    mean_po_amount : int, optional
        Mean value for normal distribution, by default 1_000_000.
    sd_po_amount : int, optional
        Standard deviation value for normal distribution, by default 250_000.
    mean_inv_line_amount : int, optional
        The mean value of invoice line items, by default 5_000.
    sd_inv_line_amount : int, optional
        The standard deviation of invoice line items, by default 4_000.
    min_year : int, optional
        The minimum possible year, by default 2000.
    max_year : int, optional
        The maximum possible year, by default 2020.
    n_invoice : int, optional
        The number of fake invoices to generate, by default 250.
    n_line_item : int, optional
        The number of fake invoice line items to generate, by default 5_000.

    Returns
    -------
    dict
        A dictionary containing related dataframes.
    """
    vendor_table = create_vendor_table(n=n_vendors)
    vendor_ids = vendor_table["vendor_id"]
    employee_table = create_employee_table(vendor_ids, n=n_employees)
    po_table = create_po_table(vendor_ids, mean_po_amount, sd_po_amount, n=100)
    invoice_summary_table, invoice_line_item_table = create_invoice_table(
        po_table,
        mean_inv_line_amount,
        sd_inv_line_amount,
        min_year,
        max_year,
        n_invoice,
        n_line_item,
    )
    x = {
        "vendor_table": vendor_table,
        "po_table": po_table,
        "invoice_summary_table": invoice_summary_table,
        "invoice_line_item_table": invoice_line_item_table,
        "employee_table": employee_table,
        "contract_table": None,
        "rate_sheet_table": None,
        "timesheet_table": None,
    }
    return x


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydatafaker']

package_data = \
{'': ['*']}

install_requires = \
['faker>=4.1.2,<5.0.0',
 'pandas>=1.1.1,<2.0.0',
 'sphinx-rtd-theme>=0.5.0,<0.6.0']

setup_kwargs = {
    'name': 'pydatafaker',
    'version': '0.1.1a2',
    'description': 'A python package to create fake data with relationships between tables.',
    'long_description': "\n# PyDataFaker\n\nA python package to create fake data with relationships between tables.\n\nCreating fake data can be useful for many different applications. Python\nalready has a great package for creating fake data called Faker\n<https://faker.readthedocs.io/en/master/>. Faker is great for creating\nindividual fake units of data, but it can be time consuming to create\nmore complicated fake data that is actually related to one another.\n\nImagine you are developing a new enterprise resource planning (ERP)\nsoftware to challenge SAP. You may need to create some fake data to test\nyour application. You will need an invoice table, a vendor listing,\npurchase order table, and more. PyDataFaker allows your to quickly\ncreate these tables and generates relationships between them\\!\n\nPyDataFaker is currently under development. At this time it is possible\nto create the following entities:\n\n  - Business: create a fake business with common ERP like tables\n\nMore entities are currently being developed. If you have any ideas of\nadditional entities that should be included please submit an issue here:\n<https://github.com/SamEdwardes/pydatafaker/issues>.\n\n## Table of contents\n\n  - [Installation](#installation)\n  - [Documentation](#documentation)\n  - [Usage](#usage)\n  - [Contributing](#contributing)\n  - [Credits](#credits)\n  - [Reminders for developers](#reminders-for-developers)\n\n## Installation\n\n``` bash\npip install pydatafaker\n```\n\n## Documentation\n\nDocumentation can be found at\n<https://pydatafaker.readthedocs.io/en/latest/index.html>.\n\n## Usage\n\n### Business\n\nGenerate fake business data that could be used to populate an ERP tool.\n\n``` python\nimport pandas as pd\nfrom pydatafaker import business\nbiz =  business.create_business()\n```\n\n`business.create_business()` returns a dictionary containing all of the\nrelated tables.\n\n``` python\nbiz.keys()\n```\n\n    ## dict_keys(['vendor_table', 'po_table', 'invoice_summary_table', 'invoice_line_item_table', 'employee_table', 'contract_table', 'rate_sheet_table', 'timesheet_table'])\n\nEach value inside the dictionary contains a Pandas DataFrame.\n\n``` python\nbiz['invoice_summary_table']\n```\n\n    ##     invoice_id  amount invoice_date     po_id     vendor_id\n    ## 0    inv_00001   85313   2006-05-18  po_00074  vendor_00020\n    ## 1    inv_00002  102511   2010-03-19  po_00017  vendor_00048\n    ## 2    inv_00003  116998   2004-04-04  po_00013  vendor_00006\n    ## 3    inv_00004   91595   2010-03-05  po_00023  vendor_00003\n    ## 4    inv_00005  127056   2014-05-13  po_00060  vendor_00028\n    ## ..         ...     ...          ...       ...           ...\n    ## 245  inv_00246   54936   2018-11-09  po_00071  vendor_00015\n    ## 246  inv_00247   97616   2004-03-25  po_00071  vendor_00015\n    ## 247  inv_00248   98365   2000-09-04  po_00064  vendor_00010\n    ## 248  inv_00249   74361   2005-09-02  po_00052  vendor_00032\n    ## 249  inv_00250   68888   2008-07-07  po_00073  vendor_00097\n    ## \n    ## [250 rows x 5 columns]\n\nTables can be joined together to add additional details.\n\n``` python\ninvoice_summary = biz['invoice_summary_table']\nvendors = biz['vendor_table']\n\npd.merge(invoice_summary, vendors, how='left', on='vendor_id')\n```\n\n    ##     invoice_id  amount  ...                   phone                       email\n    ## 0    inv_00001   85313  ...           (919)472-5788        daniel91@example.com\n    ## 1    inv_00002  102511  ...       (178)697-5211x058    seancastillo@example.org\n    ## 2    inv_00003  116998  ...            932.430.6920     mooreamanda@example.org\n    ## 3    inv_00004   91595  ...        958-198-9444x355      samantha22@example.com\n    ## 4    inv_00005  127056  ...  001-566-535-4000x26384     frankbarron@example.com\n    ## ..         ...     ...  ...                     ...                         ...\n    ## 245  inv_00246   54936  ...       135-151-8494x2791        marvin72@example.org\n    ## 246  inv_00247   97616  ...       135-151-8494x2791        marvin72@example.org\n    ## 247  inv_00248   98365  ...            642-833-5079  dianahernandez@example.net\n    ## 248  inv_00249   74361  ...       (794)308-1258x383     billyvaldez@example.net\n    ## 249  inv_00250   68888  ...   001-291-455-4032x9171    adamsjasmine@example.com\n    ## \n    ## [250 rows x 10 columns]\n\n## Contributing\n\nPlease see [docs/source/contributing.rst](docs/source/contributing.rst).\n\n## Credits\n\nDeveloped by:\n\n  - Sam Edwardes\n\nLogo:\n\n  - Icon made by [Freepik](https://www.flaticon.com/authors/freepik)\n    from [www.flaticon.com](https://www.flaticon.com/)\n  - Front from\n    [fontmeme.com/retro-fonts/](https://fontmeme.com/retro-fonts/)\n  - Logo generated using [logomakr.com](logomakr.com/7scB4)\n",
    'author': 'Sam Edwardes',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/SamEdwardes/pydatafaker',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
